@@ -61,6 +61,9 @@ order to avoid confusion with operating system threads.
 #endif
 
 
+typedef uint8_t RE_SPLIT_ID_TYPE;
+
+
 typedef struct _RE_REPEAT_ARGS
 {
   uint16_t  min;
@@ -86,51 +89,11 @@ typedef struct _RE_EMIT_CONTEXT {
 } RE_EMIT_CONTEXT;
 
 
-typedef struct _RE_FIBER
-{
-  const uint8_t* ip;    // instruction pointer
-  int32_t  sp;          // stack pointer
-  int32_t  rc;          // repeat counter
-
-  uint16_t stack[RE_MAX_STACK];
-
-  struct _RE_FIBER* prev;
-  struct _RE_FIBER* next;
-
-} RE_FIBER;
-
-
-typedef struct _RE_FIBER_LIST
-{
-  RE_FIBER* head;
-  RE_FIBER* tail;
-
-} RE_FIBER_LIST;
-
-
-typedef struct _RE_FIBER_POOL
-{
-  int fiber_count;
-  RE_FIBER_LIST fibers;
-
-} RE_FIBER_POOL;
-
-
-typedef struct _RE_THREAD_STORAGE
-{
-  RE_FIBER_POOL fiber_pool;
-
-} RE_THREAD_STORAGE;
-
-
-YR_THREAD_STORAGE_KEY thread_storage_key = 0;
-
-
 #define CHAR_IN_CLASS(cls, chr)  \
   ((cls)[(chr) / 8] & 1 << ((chr) % 8))
 
 
-static int _yr_re_is_char_in_class(
+static bool _yr_re_is_char_in_class(
     RE_CLASS* re_class,
     uint8_t chr,
     int case_insensitive)
@@ -147,7 +110,7 @@ static int _yr_re_is_char_in_class(
 }
 
 
-static int _yr_re_is_word_char(
+static bool _yr_re_is_word_char(
     const uint8_t* input,
     uint8_t character_size)
 {
@@ -160,70 +123,6 @@ static int _yr_re_is_word_char(
 }
 
 
-//
-// yr_re_initialize
-//
-// Should be called by main thread before any other
-// function from this module.
-//
-
-int yr_re_initialize(void)
-{
-  return yr_thread_storage_create(&thread_storage_key);
-}
-
-//
-// yr_re_finalize
-//
-// Should be called by main thread after every other thread
-// stopped using functions from this module.
-//
-
-int yr_re_finalize(void)
-{
-  yr_thread_storage_destroy(&thread_storage_key);
-
-  thread_storage_key = 0;
-  return ERROR_SUCCESS;
-}
-
-//
-// yr_re_finalize_thread
-//
-// Should be called by every thread using this module
-// before exiting.
-//
-
-int yr_re_finalize_thread(void)
-{
-  RE_FIBER* fiber;
-  RE_FIBER* next_fiber;
-  RE_THREAD_STORAGE* storage;
-
-  if (thread_storage_key != 0)
-    storage = (RE_THREAD_STORAGE*) yr_thread_storage_get_value(
-        &thread_storage_key);
-  else
-    return ERROR_SUCCESS;
-
-  if (storage != NULL)
-  {
-    fiber = storage->fiber_pool.fibers.head;
-
-    while (fiber != NULL)
-    {
-      next_fiber = fiber->next;
-      yr_free(fiber);
-      fiber = next_fiber;
-    }
-
-    yr_free(storage);
-  }
-
-  return yr_thread_storage_set_value(&thread_storage_key, NULL);
-}
-
-
 RE_NODE* yr_re_node_create(
     int type,
     RE_NODE* left,
@@ -236,7 +135,7 @@ RE_NODE* yr_re_node_create(
     result->type = type;
     result->left = left;
     result->right = right;
-    result->greedy = TRUE;
+    result->greedy = true;
     result->forward_code = NULL;
     result->backward_code = NULL;
   }
@@ -351,7 +250,7 @@ int yr_re_compile(
       yr_re_ast_destroy(re_ast));
 
   FAIL_ON_ERROR_WITH_CLEANUP(
-      yr_re_ast_emit_code(re_ast, code_arena, FALSE),
+      yr_re_ast_emit_code(re_ast, code_arena, false),
       yr_re_ast_destroy(re_ast));
 
   yr_re_ast_destroy(re_ast);
@@ -366,20 +265,23 @@ int yr_re_compile(
 // Verifies if the target string matches the pattern
 //
 // Args:
-//    RE* re          -  A pointer to a compiled regexp
-//    char* target    -  Target string
+//    YR_SCAN_CONTEXT* context  - Scan context
+//    RE* re                    -  A pointer to a compiled regexp
+//    char* target              -  Target string
 //
 // Returns:
 //    See return codes for yr_re_exec
 
 
 int yr_re_match(
+    YR_SCAN_CONTEXT* context,
     RE* re,
     const char* target)
 {
   int result;
 
   yr_re_exec(
+      context,
       re->code,
       (uint8_t*) target,
       strlen(target),
@@ -411,7 +313,6 @@ SIZED_STRING* yr_re_ast_extract_literal(
   RE_NODE* node = re_ast->root_node;
 
   int i, length = 0;
-  char tmp;
 
   while (node != NULL)
   {
@@ -435,26 +336,20 @@ SIZED_STRING* yr_re_ast_extract_literal(
   if (string == NULL)
     return NULL;
 
-  string->length = 0;
-
+  string->length = length;
   node = re_ast->root_node;
 
-  while (node->type == RE_NODE_CONCAT)
+  // The root node is the end of the string. So let's fill it up backwards.
+  for (i = length - 1; i > 0; i--)
   {
-    string->c_string[string->length++] = node->right->value;
+    string->c_string[i] = node->right->value;
     node = node->left;
   }
 
-  string->c_string[string->length++] = node->value;
+  if (length > 0)
+    string->c_string[0] = node->value;
 
-  // The string ends up reversed. Reverse it back to its original value.
-
-  for (i = 0; i < length / 2; i++)
-  {
-    tmp = string->c_string[i];
-    string->c_string[i] = string->c_string[length - i - 1];
-    string->c_string[length - i - 1] = tmp;
-  }
+  assert(node == NULL || node->type == RE_NODE_LITERAL);
 
   return string;
 }
@@ -463,16 +358,17 @@ SIZED_STRING* yr_re_ast_extract_literal(
 int _yr_re_node_contains_dot_star(
     RE_NODE* re_node)
 {
-  if (re_node->type == RE_NODE_STAR && re_node->left->type == RE_NODE_ANY)
-    return TRUE;
+  if ((re_node->type == RE_NODE_STAR || re_node->type == RE_NODE_PLUS) &&
+      re_node->left->type == RE_NODE_ANY)
+    return true;
 
   if (re_node->left != NULL && _yr_re_node_contains_dot_star(re_node->left))
-    return TRUE;
+    return true;
 
   if (re_node->right != NULL && _yr_re_node_contains_dot_star(re_node->right))
-    return TRUE;
+    return true;
 
-  return FALSE;
+  return false;
 }
 
 
@@ -522,9 +418,9 @@ int yr_re_ast_split_at_chaining_point(
   {
     if (child->right != NULL &&
         child->right->type == RE_NODE_RANGE_ANY &&
-        child->right->greedy == FALSE &&
-        (child->right->start > STRING_CHAINING_THRESHOLD ||
-         child->right->end > STRING_CHAINING_THRESHOLD))
+        child->right->greedy == false &&
+        (child->right->start > YR_STRING_CHAINING_THRESHOLD ||
+         child->right->end > YR_STRING_CHAINING_THRESHOLD))
     {
       result = yr_re_ast_create(remainder_re_ast);
 
@@ -760,10 +656,10 @@ static int _yr_re_emit(
   size_t inst_size;
   size_t jmp_size;
 
-  int emit_split;
-  int emit_repeat;
-  int emit_prolog;
-  int emit_epilog;
+  bool emit_split;
+  bool emit_repeat;
+  bool emit_prolog;
+  bool emit_epilog;
 
   RE_REPEAT_ARGS repeat_args;
   RE_REPEAT_ARGS* repeat_start_args_addr;
@@ -1150,7 +1046,11 @@ static int _yr_re_emit(
     //
     //        3,3     X       1,1         -      X
     //        3,4     X       2,2         X      X
-    //        3,M     X       2,M-1       X      X
+    //        3,M     X       2,M-2       X      X
+    //
+    //        4,4     X       2,2         -      X
+    //        4,5     X       3,3         X      X
+    //        4,M     X       3,M-2       X      X
     //
     // The code can't consists simply in the repeat section, the prolog and
     // epilog are required because we can't have atoms pointing to code inside
@@ -1187,9 +1087,14 @@ static int _yr_re_emit(
       }
 
       if (emit_split)
+      {
         repeat_args.max--;
+      }
       else
+      {
         repeat_args.min--;
+        repeat_args.max--;
+      }
 
       repeat_args.offset = 0;
 
@@ -1334,31 +1239,6 @@ int yr_re_ast_emit_code(
 }
 
 
-static int _yr_re_alloc_storage(
-    RE_THREAD_STORAGE** storage)
-{
-  *storage = (RE_THREAD_STORAGE*) yr_thread_storage_get_value(
-      &thread_storage_key);
-
-  if (*storage == NULL)
-  {
-    *storage = (RE_THREAD_STORAGE*) yr_malloc(sizeof(RE_THREAD_STORAGE));
-
-    if (*storage == NULL)
-      return ERROR_INSUFFICIENT_MEMORY;
-
-    (*storage)->fiber_pool.fiber_count = 0;
-    (*storage)->fiber_pool.fibers.head = NULL;
-    (*storage)->fiber_pool.fibers.tail = NULL;
-
-    FAIL_ON_ERROR(
-        yr_thread_storage_set_value(&thread_storage_key, *storage));
-  }
-
-  return ERROR_SUCCESS;
-}
-
-
 static int _yr_re_fiber_create(
     RE_FIBER_POOL* fiber_pool,
     RE_FIBER** new_fiber)
@@ -1446,7 +1326,7 @@ static int _yr_re_fiber_exists(
   int i;
 
   if (last_fiber == NULL)
-    return FALSE;
+    return false;
 
   while (fiber != last_fiber->next)
   {
@@ -1454,25 +1334,25 @@ static int _yr_re_fiber_exists(
         fiber->sp == target_fiber->sp &&
         fiber->rc == target_fiber->rc)
     {
-      equal_stacks = TRUE;
+      equal_stacks = true;
 
       for (i = 0; i <= fiber->sp; i++)
       {
         if (fiber->stack[i] != target_fiber->stack[i])
         {
-          equal_stacks = FALSE;
+          equal_stacks = false;
           break;
         }
       }
 
       if (equal_stacks)
-        return TRUE;
+        return true;
     }
 
     fiber = fiber->next;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1658,7 +1538,7 @@ static int _yr_re_fiber_sync(
       case RE_OPCODE_SPLIT_B:
 
         split_id = *(RE_SPLIT_ID_TYPE*)(fiber->ip + 1);
-        split_already_executed = FALSE;
+        split_already_executed = false;
 
         for (splits_executed_idx = 0;
              splits_executed_idx < splits_executed_count;
@@ -1666,7 +1546,7 @@ static int _yr_re_fiber_sync(
         {
           if (split_id == splits_executed[splits_executed_idx])
           {
-            split_already_executed = TRUE;
+            split_already_executed = true;
             break;
           }
         }
@@ -1848,6 +1728,7 @@ static int _yr_re_fiber_sync(
 //                              input
 //
 // Args:
+//   YR_SCAN_CONTEXT *context         - Scan context.
 //   const uint8_t* code              - Regexp code be executed
 //   const uint8_t* input             - Pointer to input data
 //   size_t input_forwards_size       - Number of accessible bytes starting at
@@ -1871,6 +1752,7 @@ static int _yr_re_fiber_sync(
 //    ERROR_SUCCESS or any other error code.
 
 int yr_re_exec(
+    YR_SCAN_CONTEXT* context,
     const uint8_t* code,
     const uint8_t* input_data,
     size_t input_forwards_size,
@@ -1888,7 +1770,6 @@ int yr_re_exec(
   uint8_t character_size;
 
   RE_FIBER_LIST fibers;
-  RE_THREAD_STORAGE* storage;
   RE_FIBER* fiber;
   RE_FIBER* next_fiber;
 
@@ -1916,9 +1797,6 @@ int yr_re_exec(
   if (matches != NULL)
     *matches = -1;
 
-  if (_yr_re_alloc_storage(&storage) != ERROR_SUCCESS)
-    return -2;
-
   if (flags & RE_FLAGS_WIDE)
     character_size = 2;
   else
@@ -1945,15 +1823,15 @@ int yr_re_exec(
   max_bytes_matched = max_bytes_matched - max_bytes_matched % character_size;
   bytes_matched = 0;
 
-  FAIL_ON_ERROR(_yr_re_fiber_create(&storage->fiber_pool, &fiber));
+  FAIL_ON_ERROR(_yr_re_fiber_create(&context->re_fiber_pool, &fiber));
 
   fiber->ip = code;
   fibers.head = fiber;
   fibers.tail = fiber;
 
   FAIL_ON_ERROR_WITH_CLEANUP(
-      _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber),
-      _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+      _yr_re_fiber_sync(&fibers, &context->re_fiber_pool, fiber),
+      _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
 
   while (fibers.head != NULL)
   {
@@ -1964,7 +1842,7 @@ int yr_re_exec(
       next_fiber = fiber->next;
 
       if (_yr_re_fiber_exists(&fibers, fiber, fiber->prev))
-        _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
+        _yr_re_fiber_kill(&fibers, &context->re_fiber_pool, fiber);
 
       fiber = next_fiber;
     }
@@ -2057,10 +1935,10 @@ int yr_re_exec(
             case '\n':
             case '\v':
             case '\f':
-              match = TRUE;
+              match = true;
               break;
             default:
-              match = FALSE;
+              match = false;
           }
 
           if (*ip == RE_OPCODE_NON_SPACE)
@@ -2089,11 +1967,11 @@ int yr_re_exec(
 
           if (bytes_matched == 0 && input_backwards_size < character_size)
           {
-            match = TRUE;
+            match = true;
           }
           else if (bytes_matched >= max_bytes_matched)
           {
-            match = TRUE;
+            match = true;
           }
           else
           {
@@ -2147,7 +2025,7 @@ int yr_re_exec(
                         bytes_matched,
                         flags,
                         callback_args),
-                    _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+                    _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
               }
               else
               {
@@ -2157,7 +2035,7 @@ int yr_re_exec(
                         bytes_matched,
                         flags,
                         callback_args),
-                    _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+                    _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
               }
             }
 
@@ -2171,31 +2049,31 @@ int yr_re_exec(
           break;
 
         default:
-          assert(FALSE);
+          assert(false);
       }
 
       switch (action)
       {
         case ACTION_KILL:
-          fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
+          fiber = _yr_re_fiber_kill(&fibers, &context->re_fiber_pool, fiber);
           break;
 
         case ACTION_KILL_TAIL:
-          _yr_re_fiber_kill_tail(&fibers, &storage->fiber_pool, fiber);
+          _yr_re_fiber_kill_tail(&fibers, &context->re_fiber_pool, fiber);
           fiber = NULL;
           break;
 
         case ACTION_CONTINUE:
           FAIL_ON_ERROR_WITH_CLEANUP(
-              _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber),
-              _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+              _yr_re_fiber_sync(&fibers, &context->re_fiber_pool, fiber),
+              _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
           break;
 
         default:
           next_fiber = fiber->next;
           FAIL_ON_ERROR_WITH_CLEANUP(
-              _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber),
-              _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+              _yr_re_fiber_sync(&fibers, &context->re_fiber_pool, fiber),
+              _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
           fiber = next_fiber;
       }
     }
@@ -2206,23 +2084,36 @@ int yr_re_exec(
     if (flags & RE_FLAGS_SCAN && bytes_matched < max_bytes_matched)
     {
       FAIL_ON_ERROR_WITH_CLEANUP(
-          _yr_re_fiber_create(&storage->fiber_pool, &fiber),
-          _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+          _yr_re_fiber_create(&context->re_fiber_pool, &fiber),
+          _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
 
       fiber->ip = code;
       _yr_re_fiber_append(&fibers, fiber);
 
       FAIL_ON_ERROR_WITH_CLEANUP(
-          _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber),
-          _yr_re_fiber_kill_all(&fibers, &storage->fiber_pool));
+          _yr_re_fiber_sync(&fibers, &context->re_fiber_pool, fiber),
+          _yr_re_fiber_kill_all(&fibers, &context->re_fiber_pool));
     }
   }
 
   return ERROR_SUCCESS;
 }
 
+//
+// yr_re_fast_exec
+//
+// This function replaces yr_re_exec for regular expressions marked with flag
+// RE_FLAGS_FAST_REGEXP. These are regular expression whose code contain only
+// the following operations: RE_OPCODE_LITERAL, RE_OPCODE_MASKED_LITERAL,
+// RE_OPCODE_ANY, RE_OPCODE_REPEAT_ANY_UNGREEDY and RE_OPCODE_MATCH. Some
+// examples of regular expressions that can be executed with this function are:
+//
+//  /foobar/
+//  /foo.*?bar/
+//
 
 int yr_re_fast_exec(
+    YR_SCAN_CONTEXT* context,
     const uint8_t* code,
     const uint8_t* input_data,
     size_t input_forwards_size,
@@ -2234,9 +2125,9 @@ int yr_re_fast_exec(
 {
   RE_REPEAT_ANY_ARGS* repeat_any_args;
 
-  const uint8_t* code_stack[MAX_FAST_RE_STACK];
-  const uint8_t* input_stack[MAX_FAST_RE_STACK];
-  int matches_stack[MAX_FAST_RE_STACK];
+  const uint8_t* code_stack[YR_MAX_FAST_RE_STACK];
+  const uint8_t* input_stack[YR_MAX_FAST_RE_STACK];
+  int matches_stack[YR_MAX_FAST_RE_STACK];
 
   const uint8_t* input = input_data;
   const uint8_t* next_input;
@@ -2273,7 +2164,7 @@ int yr_re_fast_exec(
     ip = code_stack[sp];
     input = input_stack[sp];
     bytes_matched = matches_stack[sp];
-    stop = FALSE;
+    stop = false;
 
     while (!stop)
     {
@@ -2313,7 +2204,7 @@ int yr_re_fast_exec(
           }
           else
           {
-            stop = TRUE;
+            stop = true;
           }
 
           break;
@@ -2331,7 +2222,7 @@ int yr_re_fast_exec(
           }
           else
           {
-            stop = TRUE;
+            stop = true;
           }
 
           break;
@@ -2360,8 +2251,8 @@ int yr_re_fast_exec(
                 (*(next_opcode) == RE_OPCODE_LITERAL &&
                  *(next_opcode + 1) == *next_input))
             {
-              if (sp >= MAX_FAST_RE_STACK)
-                return -4;
+              if (sp >= YR_MAX_FAST_RE_STACK)
+                return ERROR_TOO_MANY_RE_FIBERS;
 
               code_stack[sp] = next_opcode;
               input_stack[sp] = next_input;
@@ -2378,7 +2269,7 @@ int yr_re_fast_exec(
           break;
 
         default:
-          assert(FALSE);
+          assert(false);
       }
     }
   }
@@ -2473,7 +2364,7 @@ static void _yr_re_print_node(
   case RE_NODE_CLASS:
     printf("Class(");
     for (i = 0; i < 256; i++)
-      if (_yr_re_is_char_in_class(re_node->re_class, i, FALSE))
+      if (_yr_re_is_char_in_class(re_node->re_class, i, false))
         printf("%02X,", i);
     printf(")");
     break;
