@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.0.5.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.4"
+#define YYBISON_VERSION "3.0.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -99,7 +99,7 @@
 #define fail_if(x, error) \
     if (x) \
     { \
-      lex_env->last_error_code = error; \
+      lex_env->last_error = error; \
       YYABORT; \
     } \
 
@@ -500,9 +500,9 @@ static const yytype_uint8 yytranslate[] =
 static const yytype_uint16 yyrline[] =
 {
        0,   112,   112,   117,   121,   125,   141,   163,   167,   184,
-     198,   214,   228,   244,   267,   291,   313,   336,   340,   346,
-     352,   358,   367,   375,   381,   389,   395,   401,   407,   413,
-     419,   425
+     198,   214,   228,   244,   266,   289,   311,   334,   338,   344,
+     350,   356,   365,   373,   379,   387,   393,   399,   405,   411,
+     417,   423
 };
 #endif
 
@@ -982,6 +982,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
       case N:                               \
         yyformat = S;                       \
       break
+    default: /* Avoid compiler warnings. */
       YYCASE_(0, YY_("syntax error"));
       YYCASE_(1, YY_("syntax error, unexpected %s"));
       YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
@@ -1047,33 +1048,33 @@ yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, void *yyscanner, R
   switch (yytype)
     {
           case 6: /* _CLASS_  */
-#line 104 "re_grammar.y" /* yacc.c:1257  */
+#line 104 "re_grammar.y" /* yacc.c:1258  */
       { yr_free(((*yyvaluep).re_class)); ((*yyvaluep).re_class) = NULL; }
-#line 1053 "re_grammar.c" /* yacc.c:1257  */
+#line 1054 "re_grammar.c" /* yacc.c:1258  */
         break;
 
     case 26: /* alternative  */
-#line 105 "re_grammar.y" /* yacc.c:1257  */
+#line 105 "re_grammar.y" /* yacc.c:1258  */
       { yr_re_node_destroy(((*yyvaluep).re_node)); ((*yyvaluep).re_node) = NULL; }
-#line 1059 "re_grammar.c" /* yacc.c:1257  */
+#line 1060 "re_grammar.c" /* yacc.c:1258  */
         break;
 
     case 27: /* concatenation  */
-#line 106 "re_grammar.y" /* yacc.c:1257  */
+#line 106 "re_grammar.y" /* yacc.c:1258  */
       { yr_re_node_destroy(((*yyvaluep).re_node)); ((*yyvaluep).re_node) = NULL; }
-#line 1065 "re_grammar.c" /* yacc.c:1257  */
+#line 1066 "re_grammar.c" /* yacc.c:1258  */
         break;
 
     case 28: /* repeat  */
-#line 107 "re_grammar.y" /* yacc.c:1257  */
+#line 107 "re_grammar.y" /* yacc.c:1258  */
       { yr_re_node_destroy(((*yyvaluep).re_node)); ((*yyvaluep).re_node) = NULL; }
-#line 1071 "re_grammar.c" /* yacc.c:1257  */
+#line 1072 "re_grammar.c" /* yacc.c:1258  */
         break;
 
     case 29: /* single  */
-#line 108 "re_grammar.y" /* yacc.c:1257  */
+#line 108 "re_grammar.y" /* yacc.c:1258  */
       { yr_re_node_destroy(((*yyvaluep).re_node)); ((*yyvaluep).re_node) = NULL; }
-#line 1077 "re_grammar.c" /* yacc.c:1257  */
+#line 1078 "re_grammar.c" /* yacc.c:1258  */
         break;
 
 
@@ -1339,24 +1340,24 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 113 "re_grammar.y" /* yacc.c:1646  */
+#line 113 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast = yyget_extra(yyscanner);
         re_ast->root_node = (yyvsp[0].re_node);
       }
-#line 1348 "re_grammar.c" /* yacc.c:1646  */
+#line 1349 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 4:
-#line 122 "re_grammar.y" /* yacc.c:1646  */
+#line 122 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = (yyvsp[0].re_node);
       }
-#line 1356 "re_grammar.c" /* yacc.c:1646  */
+#line 1357 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 5:
-#line 126 "re_grammar.y" /* yacc.c:1646  */
+#line 126 "re_grammar.y" /* yacc.c:1663  */
     {
         mark_as_not_fast_regexp();
 
@@ -1372,11 +1373,11 @@ yyreduce:
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1376 "re_grammar.c" /* yacc.c:1646  */
+#line 1377 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 6:
-#line 142 "re_grammar.y" /* yacc.c:1646  */
+#line 142 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_NODE* node;
 
@@ -1395,19 +1396,19 @@ yyreduce:
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1399 "re_grammar.c" /* yacc.c:1646  */
+#line 1400 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 7:
-#line 164 "re_grammar.y" /* yacc.c:1646  */
+#line 164 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = (yyvsp[0].re_node);
       }
-#line 1407 "re_grammar.c" /* yacc.c:1646  */
+#line 1408 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 8:
-#line 168 "re_grammar.y" /* yacc.c:1646  */
+#line 168 "re_grammar.y" /* yacc.c:1663  */
     {
         fail_if_too_many_ast_levels({
           yr_re_node_destroy((yyvsp[-1].re_node));
@@ -1421,11 +1422,11 @@ yyreduce:
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1425 "re_grammar.c" /* yacc.c:1646  */
+#line 1426 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 9:
-#line 185 "re_grammar.y" /* yacc.c:1646  */
+#line 185 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast;
 
@@ -1439,11 +1440,11 @@ yyreduce:
         destroy_node_if((yyval.re_node) == NULL, (yyvsp[-1].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1443 "re_grammar.c" /* yacc.c:1646  */
+#line 1444 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 10:
-#line 199 "re_grammar.y" /* yacc.c:1646  */
+#line 199 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast;
 
@@ -1457,13 +1458,13 @@ yyreduce:
         destroy_node_if((yyval.re_node) == NULL, (yyvsp[-2].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
 
-        (yyval.re_node)->greedy = FALSE;
+        (yyval.re_node)->greedy = false;
       }
-#line 1463 "re_grammar.c" /* yacc.c:1646  */
+#line 1464 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 11:
-#line 215 "re_grammar.y" /* yacc.c:1646  */
+#line 215 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast;
 
@@ -1477,11 +1478,11 @@ yyreduce:
         destroy_node_if((yyval.re_node) == NULL, (yyvsp[-1].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1481 "re_grammar.c" /* yacc.c:1646  */
+#line 1482 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 12:
-#line 229 "re_grammar.y" /* yacc.c:1646  */
+#line 229 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast;
 
@@ -1495,13 +1496,13 @@ yyreduce:
         destroy_node_if((yyval.re_node) == NULL, (yyvsp[-2].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
 
-        (yyval.re_node)->greedy = FALSE;
+        (yyval.re_node)->greedy = false;
       }
-#line 1501 "re_grammar.c" /* yacc.c:1646  */
+#line 1502 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 13:
-#line 245 "re_grammar.y" /* yacc.c:1646  */
+#line 245 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast = yyget_extra(yyscanner);
         re_ast->flags |= RE_FLAGS_GREEDY;
@@ -1509,7 +1510,7 @@ yyreduce:
         if ((yyvsp[-1].re_node)->type == RE_NODE_ANY)
         {
           (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE_ANY, NULL, NULL);
-          destroy_node_if(TRUE, (yyvsp[-1].re_node));
+          destroy_node_if(true, (yyvsp[-1].re_node));
         }
         else
         {
@@ -1518,17 +1519,16 @@ yyreduce:
           destroy_node_if((yyval.re_node) == NULL, (yyvsp[-1].re_node));
         }
 
-        destroy_node_if((yyval.re_node) == NULL, (yyvsp[-1].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
 
         (yyval.re_node)->start = 0;
         (yyval.re_node)->end = 1;
       }
-#line 1528 "re_grammar.c" /* yacc.c:1646  */
+#line 1528 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 14:
-#line 268 "re_grammar.y" /* yacc.c:1646  */
+#line 267 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast = yyget_extra(yyscanner);
         re_ast->flags |= RE_FLAGS_UNGREEDY;
@@ -1536,7 +1536,7 @@ yyreduce:
         if ((yyvsp[-2].re_node)->type == RE_NODE_ANY)
         {
           (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE_ANY, NULL, NULL);
-          destroy_node_if(TRUE, (yyvsp[-2].re_node));
+          destroy_node_if(true, (yyvsp[-2].re_node));
         }
         else
         {
@@ -1545,18 +1545,17 @@ yyreduce:
           destroy_node_if((yyval.re_node) == NULL, (yyvsp[-2].re_node));
         }
 
-        destroy_node_if((yyval.re_node) == NULL, (yyvsp[-2].re_node));
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
 
         (yyval.re_node)->start = 0;
         (yyval.re_node)->end = 1;
-        (yyval.re_node)->greedy = FALSE;
+        (yyval.re_node)->greedy = false;
       }
-#line 1556 "re_grammar.c" /* yacc.c:1646  */
+#line 1555 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 15:
-#line 292 "re_grammar.y" /* yacc.c:1646  */
+#line 290 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast = yyget_extra(yyscanner);
         re_ast->flags |= RE_FLAGS_GREEDY;
@@ -1564,7 +1563,7 @@ yyreduce:
         if ((yyvsp[-1].re_node)->type == RE_NODE_ANY)
         {
           (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE_ANY, NULL, NULL);
-          destroy_node_if(TRUE, (yyvsp[-1].re_node));
+          destroy_node_if(true, (yyvsp[-1].re_node));
         }
         else
         {
@@ -1578,11 +1577,11 @@ yyreduce:
         (yyval.re_node)->start = (yyvsp[0].range) & 0xFFFF;;
         (yyval.re_node)->end = (yyvsp[0].range) >> 16;;
       }
-#line 1582 "re_grammar.c" /* yacc.c:1646  */
+#line 1581 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 16:
-#line 314 "re_grammar.y" /* yacc.c:1646  */
+#line 312 "re_grammar.y" /* yacc.c:1663  */
     {
         RE_AST* re_ast = yyget_extra(yyscanner);
         re_ast->flags |= RE_FLAGS_UNGREEDY;
@@ -1590,7 +1589,7 @@ yyreduce:
         if ((yyvsp[-2].re_node)->type == RE_NODE_ANY)
         {
           (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE_ANY, NULL, NULL);
-          destroy_node_if(TRUE, (yyvsp[-2].re_node));
+          destroy_node_if(true, (yyvsp[-2].re_node));
         }
         else
         {
@@ -1603,61 +1602,61 @@ yyreduce:
 
         (yyval.re_node)->start = (yyvsp[-1].range) & 0xFFFF;;
         (yyval.re_node)->end = (yyvsp[-1].range) >> 16;;
-        (yyval.re_node)->greedy = FALSE;
+        (yyval.re_node)->greedy = false;
       }
-#line 1609 "re_grammar.c" /* yacc.c:1646  */
+#line 1608 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 17:
-#line 337 "re_grammar.y" /* yacc.c:1646  */
+#line 335 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = (yyvsp[0].re_node);
       }
-#line 1617 "re_grammar.c" /* yacc.c:1646  */
+#line 1616 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 18:
-#line 341 "re_grammar.y" /* yacc.c:1646  */
+#line 339 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_WORD_BOUNDARY, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1627 "re_grammar.c" /* yacc.c:1646  */
+#line 1626 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 19:
-#line 347 "re_grammar.y" /* yacc.c:1646  */
+#line 345 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_NON_WORD_BOUNDARY, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1637 "re_grammar.c" /* yacc.c:1646  */
+#line 1636 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 20:
-#line 353 "re_grammar.y" /* yacc.c:1646  */
+#line 351 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_ANCHOR_START, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1647 "re_grammar.c" /* yacc.c:1646  */
+#line 1646 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 21:
-#line 359 "re_grammar.y" /* yacc.c:1646  */
+#line 357 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_ANCHOR_END, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1657 "re_grammar.c" /* yacc.c:1646  */
+#line 1656 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 22:
-#line 368 "re_grammar.y" /* yacc.c:1646  */
+#line 366 "re_grammar.y" /* yacc.c:1663  */
     {
         fail_if_too_many_ast_levels({
           yr_re_node_destroy((yyvsp[-1].re_node));
@@ -1665,21 +1664,21 @@ yyreduce:
 
         (yyval.re_node) = (yyvsp[-1].re_node);
       }
-#line 1669 "re_grammar.c" /* yacc.c:1646  */
+#line 1668 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 23:
-#line 376 "re_grammar.y" /* yacc.c:1646  */
+#line 374 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_ANY, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1679 "re_grammar.c" /* yacc.c:1646  */
+#line 1678 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 24:
-#line 382 "re_grammar.y" /* yacc.c:1646  */
+#line 380 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_LITERAL, NULL, NULL);
 
@@ -1687,71 +1686,71 @@ yyreduce:
 
         (yyval.re_node)->value = (yyvsp[0].integer);
       }
-#line 1691 "re_grammar.c" /* yacc.c:1646  */
+#line 1690 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 25:
-#line 390 "re_grammar.y" /* yacc.c:1646  */
+#line 388 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_WORD_CHAR, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1701 "re_grammar.c" /* yacc.c:1646  */
+#line 1700 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 26:
-#line 396 "re_grammar.y" /* yacc.c:1646  */
+#line 394 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_NON_WORD_CHAR, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1711 "re_grammar.c" /* yacc.c:1646  */
+#line 1710 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 27:
-#line 402 "re_grammar.y" /* yacc.c:1646  */
+#line 400 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_SPACE, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1721 "re_grammar.c" /* yacc.c:1646  */
+#line 1720 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 28:
-#line 408 "re_grammar.y" /* yacc.c:1646  */
+#line 406 "re_grammar.y" /* yacc.c:1663  */
     {
          (yyval.re_node) = yr_re_node_create(RE_NODE_NON_SPACE, NULL, NULL);
 
          fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1731 "re_grammar.c" /* yacc.c:1646  */
+#line 1730 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 29:
-#line 414 "re_grammar.y" /* yacc.c:1646  */
+#line 412 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_DIGIT, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1741 "re_grammar.c" /* yacc.c:1646  */
+#line 1740 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 30:
-#line 420 "re_grammar.y" /* yacc.c:1646  */
+#line 418 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_NON_DIGIT, NULL, NULL);
 
         fail_if((yyval.re_node) == NULL, ERROR_INSUFFICIENT_MEMORY);
       }
-#line 1751 "re_grammar.c" /* yacc.c:1646  */
+#line 1750 "re_grammar.c" /* yacc.c:1663  */
     break;
 
   case 31:
-#line 426 "re_grammar.y" /* yacc.c:1646  */
+#line 424 "re_grammar.y" /* yacc.c:1663  */
     {
         (yyval.re_node) = yr_re_node_create(RE_NODE_CLASS, NULL, NULL);
 
@@ -1759,11 +1758,11 @@ yyreduce:
 
         (yyval.re_node)->re_class = (yyvsp[0].re_class);
       }
-#line 1763 "re_grammar.c" /* yacc.c:1646  */
+#line 1762 "re_grammar.c" /* yacc.c:1663  */
     break;
 
 
-#line 1767 "re_grammar.c" /* yacc.c:1646  */
+#line 1766 "re_grammar.c" /* yacc.c:1663  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1991,5 +1990,5 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 434 "re_grammar.y" /* yacc.c:1906  */
+#line 432 "re_grammar.y" /* yacc.c:1907  */
 
