@@ -29,26 +29,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <assert.h>
 #include <string.h>
-#include <time.h>
 #include <ctype.h>
 
-#include <yara/ahocorasick.h>
-#include <yara/arena.h>
 #include <yara/error.h>
-#include <yara/exec.h>
-#include <yara/exefiles.h>
 #include <yara/filemap.h>
-#include <yara/hash.h>
 #include <yara/mem.h>
 #include <yara/proc.h>
-#include <yara/re.h>
+#include <yara/rules.h>
 #include <yara/utils.h>
-#include <yara/object.h>
 #include <yara/globals.h>
-#include <yara/libyara.h>
 #include <yara/scan.h>
-
-#include "exception.h"
+#include <yara/scanner.h>
 
 
 YR_API int yr_rules_define_integer_variable(
@@ -171,163 +162,29 @@ YR_API int yr_rules_define_string_variable(
 }
 
 
-static void _yr_rules_clean_matches(
-    YR_RULES* rules,
-    YR_SCAN_CONTEXT* context)
-{
-  YR_RULE* rule;
-  YR_STRING** string;
-
-  int tidx = context->tidx;
-
-  yr_rules_foreach(rules, rule)
-  {
-    rule->t_flags[tidx] &= ~RULE_TFLAGS_MATCH;
-    rule->ns->t_flags[tidx] &= ~NAMESPACE_TFLAGS_UNSATISFIED_GLOBAL;
-  }
-
-  if (context->matching_strings_arena != NULL)
-  {
-    string = (YR_STRING**) yr_arena_base_address(
-        context->matching_strings_arena);
-
-    while (string != NULL)
-    {
-      (*string)->matches[tidx].count = 0;
-      (*string)->matches[tidx].head = NULL;
-      (*string)->matches[tidx].tail = NULL;
-      (*string)->unconfirmed_matches[tidx].count = 0;
-      (*string)->unconfirmed_matches[tidx].head = NULL;
-      (*string)->unconfirmed_matches[tidx].tail = NULL;
-
-      string = (YR_STRING**) yr_arena_next_address(
-          context->matching_strings_arena,
-          string,
-          sizeof(YR_STRING*));
-    }
-  }
-}
-
-
 #ifdef PROFILING_ENABLED
 void yr_rules_print_profiling_info(
     YR_RULES* rules)
 {
   YR_RULE* rule;
-  YR_STRING* string;
 
-  clock_t clock_ticks;
-
-  printf("===== PROFILING INFORMATION =====\n");
+  printf("\n===== PROFILING INFORMATION =====\n\n");
 
   yr_rules_foreach(rules, rule)
   {
-    clock_ticks = rule->clock_ticks;
-
-    yr_rule_strings_foreach(rule, string)
-    {
-      clock_ticks += string->clock_ticks;
-    }
-
     printf(
-        "%s:%s: %li\n",
+        "%s:%s: %" PRIu64 " (%0.3f%%)\n",
         rule->ns->name,
         rule->identifier,
-        clock_ticks);
+        rule->time_cost,
+        (float) rule->time_cost / rules->time_cost * 100);
   }
 
-  printf("================================\n");
+  printf("\n=================================\n");
 }
 #endif
 
 
-static int _yr_rules_scan_mem_block(
-    YR_RULES* rules,
-    const uint8_t* block_data,
-    YR_MEMORY_BLOCK* block,
-    YR_SCAN_CONTEXT* context,
-    int timeout,
-    time_t start_time)
-{
-  YR_AC_TRANSITION_TABLE transition_table = rules->transition_table;
-  YR_AC_MATCH_TABLE match_table = rules->match_table;
-
-  YR_AC_MATCH* match;
-  YR_AC_TRANSITION transition;
-
-  size_t i = 0;
-  uint32_t state = YR_AC_ROOT_STATE;
-  uint16_t index;
-
-  while (i < block->size)
-  {
-    match = match_table[state].match;
-
-    while (match != NULL)
-    {
-      if (timeout > 0 && i % 4096 == 0)
-      {
-        if (difftime(time(NULL), start_time) > timeout)
-          return ERROR_SCAN_TIMEOUT;
-      }
-
-      if (match->backtrack <= i)
-      {
-        FAIL_ON_ERROR(yr_scan_verify_match(
-            context,
-            match,
-            block_data,
-            block->size,
-            block->base,
-            i - match->backtrack));
-      }
-
-      match = match->next;
-    }
-
-    index = block_data[i++] + 1;
-    transition = transition_table[state + index];
-
-    while (YR_AC_INVALID_TRANSITION(transition, index))
-    {
-      if (state != YR_AC_ROOT_STATE)
-      {
-        state = transition_table[state] >> 32;
-        transition = transition_table[state + index];
-      }
-      else
-      {
-        transition = 0;
-        break;
-      }
-    }
-
-    state = transition >> 32;
-
-  }
-
-  match = match_table[state].match;
-
-  while (match != NULL)
-  {
-    if (match->backtrack <= i)
-    {
-      FAIL_ON_ERROR(yr_scan_verify_match(
-          context,
-          match,
-          block_data,
-          block->size,
-          block->base,
-          i - match->backtrack));
-    }
-
-    match = match->next;
-  }
-
-  return ERROR_SUCCESS;
-}
-
-
 YR_API int yr_rules_scan_mem_blocks(
     YR_RULES* rules,
     YR_MEMORY_BLOCK_ITERATOR* iterator,
@@ -336,211 +193,23 @@ YR_API int yr_rules_scan_mem_blocks(
     void* user_data,
     int timeout)
 {
-  YR_EXTERNAL_VARIABLE* external;
-  YR_RULE* rule;
-  YR_SCAN_CONTEXT context;
-  YR_MEMORY_BLOCK* block;
+  YR_SCANNER* scanner;
+  int result;
 
-  time_t start_time;
+  FAIL_ON_ERROR(yr_scanner_create(rules, &scanner));
 
-  int tidx = 0;
-  int result = ERROR_SUCCESS;
+  yr_scanner_set_callback(scanner, callback, user_data);
+  yr_scanner_set_timeout(scanner, timeout);
+  yr_scanner_set_flags(scanner, flags);
 
-  block = iterator->first(iterator);
+  result = yr_scanner_scan_mem_blocks(scanner, iterator);
 
-  if (block == NULL)
-    return ERROR_SUCCESS;
-
-  yr_mutex_lock(&rules->mutex);
-
-  while (tidx < MAX_THREADS && YR_BITARRAY_TEST(rules->tidx_mask, tidx))
-  {
-    tidx++;
-  }
-
-  if (tidx < MAX_THREADS)
-    YR_BITARRAY_SET(rules->tidx_mask, tidx);
-  else
-    result = ERROR_TOO_MANY_SCAN_THREADS;
-
-  yr_mutex_unlock(&rules->mutex);
-
-  if (result != ERROR_SUCCESS)
-    return result;
-
-  context.tidx = tidx;
-  context.flags = flags;
-  context.callback = callback;
-  context.user_data = user_data;
-  context.file_size = block->size;
-  context.iterator = iterator;
-  context.entry_point = UNDEFINED;
-  context.objects_table = NULL;
-  context.matches_arena = NULL;
-  context.matching_strings_arena = NULL;
-
-  yr_set_tidx(tidx);
-
-  result = yr_arena_create(1024, 0, &context.matches_arena);
-
-  if (result != ERROR_SUCCESS)
-    goto _exit;
-
-  result = yr_arena_create(8, 0, &context.matching_strings_arena);
-
-  if (result != ERROR_SUCCESS)
-    goto _exit;
-
-  result = yr_hash_table_create(64, &context.objects_table);
-
-  if (result != ERROR_SUCCESS)
-    goto _exit;
-
-  external = rules->externals_list_head;
-
-  while (!EXTERNAL_VARIABLE_IS_NULL(external))
-  {
-    YR_OBJECT* object;
-
-    result = yr_object_from_external_variable(
-        external,
-        &object);
-
-    if (result == ERROR_SUCCESS)
-      result = yr_hash_table_add(
-          context.objects_table,
-          external->identifier,
-          NULL,
-          (void*) object);
-
-    if (result != ERROR_SUCCESS)
-      goto _exit;
-
-    external++;
-  }
-
-  start_time = time(NULL);
-
-  while (block != NULL)
-  {
-    const uint8_t* data = block->fetch_data(block);
-
-    // fetch may fail
-    if (data == NULL)
-    {
-      block = iterator->next(iterator);
-      continue;
-    }
-
-    if (context.entry_point == UNDEFINED)
-    {
-      YR_TRYCATCH(
-        !(flags & SCAN_FLAGS_NO_TRYCATCH),
-        {
-          if (flags & SCAN_FLAGS_PROCESS_MEMORY)
-            context.entry_point = yr_get_entry_point_address(
-                data,
-                block->size,
-                block->base);
-          else
-            context.entry_point = yr_get_entry_point_offset(
-                data,
-                block->size);
-        },{});
-    }
-
-    YR_TRYCATCH(
-      !(flags & SCAN_FLAGS_NO_TRYCATCH),
-      {
-        result = _yr_rules_scan_mem_block(
-            rules,
-            data,
-            block,
-            &context,
-            timeout,
-            start_time);
-      },{
-        result = ERROR_COULD_NOT_MAP_FILE;
-      });
-
-    if (result != ERROR_SUCCESS)
-      goto _exit;
-
-    block = iterator->next(iterator);
-  }
-
-  YR_TRYCATCH(
-    !(flags & SCAN_FLAGS_NO_TRYCATCH),
-    {
-      result = yr_execute_code(
-          rules,
-          &context,
-          timeout,
-          start_time);
-    },{
-      result = ERROR_COULD_NOT_MAP_FILE;
-    });
-
-  if (result != ERROR_SUCCESS)
-    goto _exit;
-
-  yr_rules_foreach(rules, rule)
-  {
-    int message;
-
-    if (rule->t_flags[tidx] & RULE_TFLAGS_MATCH &&
-        !(rule->ns->t_flags[tidx] & NAMESPACE_TFLAGS_UNSATISFIED_GLOBAL))
-    {
-      message = CALLBACK_MSG_RULE_MATCHING;
-    }
-    else
-    {
-      message = CALLBACK_MSG_RULE_NOT_MATCHING;
-    }
-
-    if (!RULE_IS_PRIVATE(rule))
-    {
-      switch (callback(message, rule, user_data))
-      {
-        case CALLBACK_ABORT:
-          result = ERROR_SUCCESS;
-          goto _exit;
-
-        case CALLBACK_ERROR:
-          result = ERROR_CALLBACK_ERROR;
-          goto _exit;
-      }
-    }
-  }
-
-  callback(CALLBACK_MSG_SCAN_FINISHED, NULL, user_data);
-
-_exit:
-
-  _yr_rules_clean_matches(rules, &context);
-
-  if (context.matches_arena != NULL)
-    yr_arena_destroy(context.matches_arena);
-
-  if (context.matching_strings_arena != NULL)
-    yr_arena_destroy(context.matching_strings_arena);
-
-  if (context.objects_table != NULL)
-    yr_hash_table_destroy(
-        context.objects_table,
-        (YR_HASH_TABLE_FREE_VALUE_FUNC) yr_object_destroy);
-
-  yr_mutex_lock(&rules->mutex);
-  YR_BITARRAY_UNSET(rules->tidx_mask, tidx);
-  yr_mutex_unlock(&rules->mutex);
-
-  yr_set_tidx(-1);
+  yr_scanner_destroy(scanner);
 
   return result;
 }
 
 
-
 static YR_MEMORY_BLOCK* _yr_get_first_block(
     YR_MEMORY_BLOCK_ITERATOR* iterator)
 {
@@ -705,8 +374,10 @@ YR_API int yr_rules_load_stream(
   new_rules->code_start = header->code_start;
   new_rules->externals_list_head = header->externals_list_head;
   new_rules->rules_list_head = header->rules_list_head;
-  new_rules->match_table = header->match_table;
-  new_rules->transition_table = header->transition_table;
+  new_rules->ac_match_table = header->ac_match_table;
+  new_rules->ac_transition_table = header->ac_transition_table;
+  new_rules->ac_tables_size = header->ac_tables_size;
+
   memset(new_rules->tidx_mask, 0, sizeof(new_rules->tidx_mask));
 
   FAIL_ON_ERROR_WITH_CLEANUP(
@@ -748,7 +419,7 @@ YR_API int yr_rules_save_stream(
 {
   int i;
 
-  for (i = 0; i < YR_BITARRAY_NCHARS(MAX_THREADS); ++i)
+  for (i = 0; i < YR_BITARRAY_NCHARS(YR_MAX_THREADS); ++i)
     assert(rules->tidx_mask[i] == 0);
 
   return yr_arena_save_stream(rules->arena, stream);
@@ -777,6 +448,90 @@ YR_API int yr_rules_save(
 }
 
 
+static int _uint32_cmp (
+    const void * a,
+    const void * b)
+{
+   return (*(uint32_t*) a - *(uint32_t*) b);
+}
+
+YR_API int yr_rules_get_stats(
+    YR_RULES* rules,
+    YR_RULES_STATS *stats)
+{
+  YR_RULE* rule;
+  YR_STRING* string;
+
+  uint32_t* match_list_lengths = (uint32_t*) yr_malloc(
+      sizeof(uint32_t) * rules->ac_tables_size);
+
+  float match_list_length_sum = 0;
+  int i, c = 0;
+
+  if (match_list_lengths == NULL)
+    return ERROR_INSUFFICIENT_MEMORY;
+
+  stats->ac_tables_size = rules->ac_tables_size;
+  stats->ac_matches = 0;
+  stats->rules = 0;
+  stats->strings = 0;
+
+  for (i = 0; i < rules->ac_tables_size; i++)
+  {
+    YR_AC_MATCH* match = rules->ac_match_table[i].match;
+
+    int match_list_length = 0;
+
+    while (match != NULL)
+    {
+      match_list_length++;
+      stats->ac_matches++;
+      match = match->next;
+    }
+
+    if (i == 0)
+      stats->ac_root_match_list_length = match_list_length;
+
+    match_list_length_sum += match_list_length;
+
+    if (match_list_length > 0)
+    {
+      match_list_lengths[c] = match_list_length;
+      c++;
+    }
+  }
+
+  // sort match_list_lengths in increasing order for computing percentiles.
+  qsort(match_list_lengths, c, sizeof(match_list_lengths[0]), _uint32_cmp);
+
+  for (i = 0; i < 100; i++)
+  {
+    if (i < c)
+      stats->top_ac_match_list_lengths[i] = match_list_lengths[c-i-1];
+    else
+      stats->top_ac_match_list_lengths[i] = 0;
+  }
+
+  stats->ac_average_match_list_length = match_list_length_sum / c;
+  stats->ac_match_list_length_pctls[0] = match_list_lengths[0];
+  stats->ac_match_list_length_pctls[100] = match_list_lengths[c-1];
+
+  for (i = 1; i < 100; i++)
+    stats->ac_match_list_length_pctls[i] = match_list_lengths[(c * i) / 100];
+
+  yr_free(match_list_lengths);
+
+  yr_rules_foreach(rules, rule)
+  {
+    stats->rules++;
+    yr_rule_strings_foreach(rule, string)
+      stats->strings++;
+  }
+
+  return ERROR_SUCCESS;
+}
+
+
 YR_API int yr_rules_destroy(
     YR_RULES* rules)
 {
@@ -812,7 +567,7 @@ YR_API void yr_rule_disable(
 
 
 YR_API void yr_rule_enable(
-  YR_RULE* rule)
+    YR_RULE* rule)
 {
   YR_STRING* string;
 
