@@ -67,8 +67,8 @@ int err = 0;
 
 int main (int argc, char **argv)
 {
-  CHECK_SIZE(YR_NAMESPACE, 4 * MAX_THREADS + 8);
-  CHECK_OFFSET(YR_NAMESPACE, 4 * MAX_THREADS, name);
+  CHECK_SIZE(YR_NAMESPACE, 4 * YR_MAX_THREADS + 8);
+  CHECK_OFFSET(YR_NAMESPACE, 4 * YR_MAX_THREADS, name);
 
   CHECK_SIZE(YR_META, 32);
   CHECK_OFFSET(YR_META, 8,  integer);
@@ -79,22 +79,23 @@ int main (int argc, char **argv)
   CHECK_OFFSET(YR_MATCHES, 8,  head);
   CHECK_OFFSET(YR_MATCHES, 16, tail);
 
-  CHECK_SIZE(YR_STRING, 56 + 2 * 24 /* YR_MATCHES */ * MAX_THREADS);
+  CHECK_SIZE(YR_STRING, 64 + 2 * 24 /* YR_MATCHES */ * YR_MAX_THREADS);
   CHECK_OFFSET(YR_STRING, 4,  length);
   CHECK_OFFSET(YR_STRING, 8,  identifier);
   CHECK_OFFSET(YR_STRING, 16, string);
   CHECK_OFFSET(YR_STRING, 24, chained_to);
-  CHECK_OFFSET(YR_STRING, 32, chain_gap_min);
-  CHECK_OFFSET(YR_STRING, 36, chain_gap_max);
-  CHECK_OFFSET(YR_STRING, 40, fixed_offset);
+  CHECK_OFFSET(YR_STRING, 32, rule);
+  CHECK_OFFSET(YR_STRING, 40, chain_gap_min);
+  CHECK_OFFSET(YR_STRING, 44, chain_gap_max);
+  CHECK_OFFSET(YR_STRING, 48, fixed_offset);
 
-  CHECK_SIZE(YR_RULE, 16 + 4 * MAX_THREADS + 40);
+  CHECK_SIZE(YR_RULE, 16 + 4 * YR_MAX_THREADS + 40);
   CHECK_OFFSET(YR_RULE, 4,                        t_flags);
-  CHECK_OFFSET(YR_RULE, 8 + 4 * MAX_THREADS,      identifier);
-  CHECK_OFFSET(YR_RULE, 8 + 4 * MAX_THREADS + 8,  tags);
-  CHECK_OFFSET(YR_RULE, 8 + 4 * MAX_THREADS + 16, metas);
-  CHECK_OFFSET(YR_RULE, 8 + 4 * MAX_THREADS + 24, strings);
-  CHECK_OFFSET(YR_RULE, 8 + 4 * MAX_THREADS + 32, ns);
+  CHECK_OFFSET(YR_RULE, 8 + 4 * YR_MAX_THREADS,      identifier);
+  CHECK_OFFSET(YR_RULE, 8 + 4 * YR_MAX_THREADS + 8,  tags);
+  CHECK_OFFSET(YR_RULE, 8 + 4 * YR_MAX_THREADS + 16, metas);
+  CHECK_OFFSET(YR_RULE, 8 + 4 * YR_MAX_THREADS + 24, strings);
+  CHECK_OFFSET(YR_RULE, 8 + 4 * YR_MAX_THREADS + 32, ns);
 
   CHECK_SIZE(YR_EXTERNAL_VARIABLE, 24);
   CHECK_OFFSET(YR_EXTERNAL_VARIABLE, 8,  value.i);
@@ -108,12 +109,12 @@ int main (int argc, char **argv)
   CHECK_OFFSET(YR_AC_MATCH, 24, backward_code);
   CHECK_OFFSET(YR_AC_MATCH, 32, next);
 
-  CHECK_SIZE(YARA_RULES_FILE_HEADER, 40);
-  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 0,  rules_list_head);
+  CHECK_SIZE(YARA_RULES_FILE_HEADER, 48);
+  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 0, rules_list_head);
   CHECK_OFFSET(YARA_RULES_FILE_HEADER, 8, externals_list_head);
   CHECK_OFFSET(YARA_RULES_FILE_HEADER, 16, code_start);
-  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 24, match_table);
-  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 32, transition_table);
+  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 24, ac_match_table);
+  CHECK_OFFSET(YARA_RULES_FILE_HEADER, 32, ac_transition_table);
 
   CHECK_SIZE(SIZED_STRING, 12);
   CHECK_OFFSET(SIZED_STRING, 4, flags);
